<?php

namespace Drupal\commerce_wallee\Plugin\Commerce\PaymentGateway;

use Drupal\commerce_wallee\Plugin\Commerce\PaymentGateway\RedirectCheckoutInterface;
use Drupal\commerce_payment\Entity\PaymentInterface;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Drupal\commerce_payment\Exception\PaymentGatewayException;

use Drupal\commerce_price\Price;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_payment\PaymentMethodTypeManager;
use Drupal\commerce_payment\PaymentTypeManager;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Drupal\commerce_wallee\Services\PaymentSdk;

/**
 * Provides the Wallee offsite Checkout payment gateway.
 *
 * @CommercePaymentGateway(
 *   id = "wallee_redirect_checkout",
 *   label = @Translation("Wallee (Redirect to wallee)"),
 *   display_label = @Translation("Wallee"),
 *    forms = {
 *     "offsite-payment" = "Drupal\commerce_wallee\PluginForm\RedirectCheckoutForm",
 *   },
 *   payment_method_types = {"credit_card"},
 *   credit_card_types = {
 *     "mastercard", "visa",
 *   },
 * )
 */
class RedirectCheckout extends OffsitePaymentGatewayBase implements RedirectCheckoutInterface {

  /**
   * The logger channel factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  private $loggerChannelFactory;

  /**
   * The event dispatcher.
   *
   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
   */
  private $eventDispatcher;

  /**
   * The payment sdk.
   *
   * @var \Drupal\commerce_wallee\Services\PaymentSdk
   */
  private $paymentSdk;

  /**
   * Constructs a new Postfinance gateway.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $pluginId
   *   The plugin_id for the plugin instance.
   * @param mixed $pluginDefinition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\commerce_payment\PaymentTypeManager $paymentTypeManager
   *   The payment type manager.
   * @param \Drupal\commerce_payment\PaymentMethodTypeManager $paymentMethodTypeManager
   *   The payment method type manager.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $loggerChannelFactory
   *   The logger channel factory.
   * @param \Symfony\Component\EventDispatcher\EventDispatcherInterface $eventDispatcher
   *   The event dispatcher.
   * @param \Drupal\commerce_wallee\Services\PaymentSdk $paymentSdk
   *   The payment sdk.
   */
  public function __construct(array $configuration,
                              $pluginId,
                              $pluginDefinition,
                              EntityTypeManagerInterface $entityTypeManager,
                              PaymentTypeManager $paymentTypeManager,
                              PaymentMethodTypeManager $paymentMethodTypeManager,
                              TimeInterface $time,
                              LoggerChannelFactoryInterface $loggerChannelFactory,
                              EventDispatcherInterface $eventDispatcher,
                              PaymentSdk $paymentSdk
  ) {
    parent::__construct($configuration, $pluginId, $pluginDefinition, $entityTypeManager, $paymentTypeManager, $paymentMethodTypeManager, $time);
    $this->loggerChannelFactory = $loggerChannelFactory;
    $this->eventDispatcher = $eventDispatcher;
    $this->paymentSdk = $paymentSdk;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $pluginId, $pluginDefinition) {
    return new static(
      $configuration,
      $pluginId,
      $pluginDefinition,
      $container->get('entity_type.manager'),
      $container->get('plugin.manager.commerce_payment_type'),
      $container->get('plugin.manager.commerce_payment_method_type'),
      $container->get('datetime.time'),
      $container->get('logger.factory'),
      $container->get('event_dispatcher'),
      $container->get('commerce_wallee.payment_sdk')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['space_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API Space ID'),
      '#description' => $this->t('This is the space id from the Wallee PHP SDK.'),
      '#default_value' => $this->configuration['space_id'],
      '#required' => TRUE,
    ];

    $form['user_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API User ID'),
      '#description' => $this->t('The user id from the Wallee PHP SDK.'),
      '#default_value' => $this->configuration['user_id'],
      '#required' => TRUE,
    ];

    $form['secret'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API Secret'),
      '#description' => $this->t('The secret from the Wallee PHP SDK.'),
      '#default_value' => $this->configuration['secret'],
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);

    $values = $form_state->getValue($form['#parents']);
    $this->configuration['type'] = 'commerce_wallee';
    $this->configuration['space_id'] = $values['space_id'];
    $this->configuration['user_id'] = $values['user_id'];
    $this->configuration['secret'] = $values['secret'];
  }

  /**
   * {@inheritdoc}
   */
  public function onReturn(OrderInterface $order, Request $request) {
    parent::onReturn($order, $request);

    $paymentStorage = $this->entityTypeManager->getStorage('commerce_payment');
    $payments = $paymentStorage->loadMultipleByOrder($order);

    $success = FALSE;
    foreach ($payments as $payment) {
      $configuration = $payment->getPaymentGateway()->getPluginConfiguration();
      if (isset($configuration['type']) AND $configuration['type'] == 'commerce_wallee') {
        $transaction_id = $payment->getRemoteId();
        $transation = $this->paymentSdk->readTransaction($configuration['user_id'], $configuration['secret'], $configuration['space_id'], $transaction_id);

        // Check state of transaction.
        // More infos https://app-wallee.com/de-ch/doc/payment/transaction-process.
        if (isset($transation['state'])) {
          // State is authorized.
          $sate = NULL;
          if ($transation['state'] == 'VOIDED') {
            $sate = 'void';
          }
          if ($transation['state'] == 'AUTHORIZED') {
            $sate = 'authorize';
            $success = TRUE;
          }
          if ($transation['state'] == 'COMPLETED') {
            $sate = 'completed';
            $success = TRUE;
          }
          if ($transation['state'] == 'FULFILL') {
            $sate = 'completed';
            $success = TRUE;
          }

          // Update payment.
          if ($sate != NULL) {
            $payment->set('state', $sate);
            $payment->set('remote_state', $sate);
            $payment->save();
          }
        }
      }
    }

    if ($success == FALSE) {
      // Error on payment.
      throw new PaymentGatewayException('Payment incomplete or declined');
    }
  }

  /**
   * {@inheritdoc}
   */
  public function onCancel(OrderInterface $order, Request $request) {
    parent::onCancel($order, $request);
  }

  /**
   * {@inheritdoc}
   */
  public function onNotify(Request $request) {
    parent::onNotify($request);
  }

  /**
   * {@inheritdoc}
   */
  public function refundPayment(PaymentInterface $payment, Price $amount = NULL) {
    $this->assertPaymentState($payment, ['completed', 'partially_refunded']);

    // If not specified, refund the entire amount.
    $amount = $amount ?: $payment->getAmount();

    // Validate the requested amount.
    $this->assertRefundAmount($payment, $amount);

    $old_refunded_amount = $payment->getRefundedAmount();
    $new_refunded_amount = $old_refunded_amount->add($amount);
    if ($new_refunded_amount->lessThan($payment->getAmount())) {
      $payment->state = 'partially_refunded';
    }
    else {
      $payment->state = 'refunded';
    }

    // Call remote refund on Wallee.
    $configuration = $payment->getPaymentGateway()->getPluginConfiguration();
    $refund = $this->paymentSdk->createRefund($configuration['user_id'], $configuration['secret'], $configuration['space_id'], $amount->getNumber(), $payment->getRemoteId());
    if ($refund->getState() == 'SUCCESSFUL') {
      // Save refund
      $payment->setRefundedAmount($new_refunded_amount);
      $payment->save();

      $this->messenger()->addStatus($this->t('Refund on payment %payment', [
        '%payment' => $payment->id(),
      ]));
    }
    else {
      // Error on refund.
      $this->messenger()->addError($this->t('Error refund on payment %payment', [
        '%payment' => $payment->id(),
      ]));
    }
  }

}