<?php

namespace Drupal\commerce_wallee\Services;

use Drupal\commerce_wallee\Services\PaymentSdkInterface;

/**
 * Event handler for Wallee PHP SDK.
 */
class PaymentSdk implements PaymentSdkInterface {

  /**
   * {@inheritdoc}
   */
  private function apiClient($user_id, $secret) {
    return new \Wallee\Sdk\ApiClient($user_id, $secret);
  }

  /**
   * {@inheritdoc}
   */
  private function createLineItem($order_id, $amount) {
    $lineItem = new \Wallee\Sdk\Model\LineItemCreate();
    $lineItem->setName('Order ' . $order_id);
    $lineItem->setUniqueId('orderid-' . $order_id);
    $lineItem->setQuantity(1);
    $lineItem->setAmountIncludingTax($amount);
    $lineItem->setType(\Wallee\Sdk\Model\LineItemType::FEE);

    return $lineItem;
  }

  /**
   * Create address object.
   */
  public function createAddress($organisation, $salutation, $family_name, $given_name, $street, $postal_code, $city, $postal_state, $country, $mail, $phone) {
    $profileAddress = new \Wallee\Sdk\Model\Address();

    if ($organisation != '') {
      $profileAddress->setOrganizationName($organisation);
    }
    if ($salutation != '') {
      $profileAddress->setSalutation($salutation);
    }
    if ($family_name != '') {
      $profileAddress->setFamilyName($family_name);
    }
    if ($given_name != '') {
      $profileAddress->setGivenName($given_name);
    }
    if ($street != '') {
      $profileAddress->setStreet($street);
    }
    if ($postal_code != '') {
      $profileAddress->setPostCode($postal_code);
    }
    if ($city != '') {
      $profileAddress->setCity($city);
    }
    if ($postal_state != '') {
      $profileAddress->setPostalState($postal_state);
    }
    if ($country != '') {
      $profileAddress->setCountry($country);
    }
    if ($mail != '') {
      $profileAddress->setEmailAddress($mail);
    }
    if ($phone != '') {
      $profileAddress->setPhoneNumber($phone);
    }

    return $profileAddress;
  }

  /**
   * Create transaction.
   */
  public function createTransaction($user_id, $secret, $space_id, $return_url, $cancel_url, $currency, $order_id, $amount, $billingAddress, $shippingAddress, $environment = 'PREVIEW') {
    $transactionPayload = new \Wallee\Sdk\Model\TransactionCreate();
    $transactionPayload->setSuccessUrl($return_url);
    $transactionPayload->setFailedUrl($cancel_url);
    $transactionPayload->setEnvironment($environment);
    if ($billingAddress != '') {
      $transactionPayload->setBillingAddress($billingAddress);
    }
    if ($shippingAddress != '') {
      $transactionPayload->setShippingAddress($shippingAddress);
    }
    $transactionPayload->setCurrency($currency);
    $transactionPayload->setLanguage('DE');
    $transactionPayload->setLineItems([$this->createLineItem($order_id, $amount)]);
    $transactionPayload->setAutoConfirmationEnabled(true);

    $client = $this->apiClient($user_id, $secret);
    $transaction = $client->getTransactionService()->create($space_id, $transactionPayload);

    return [
      'transaction_id' => $transaction->getId(),
      'redirect_url' => $client->getTransactionPaymentPageService()->paymentPageUrl($space_id, $transaction->getId()),
    ];
  }

  /**
   * Read transaction.
   */
  public function readTransaction($user_id, $secret, $space_id, $transaction_id) {
    $client = $this->apiClient($user_id, $secret);

    $transaction = $client->getTransactionService()->read($space_id, $transaction_id);

    return [
      'authorization_amount' => $transaction->getAuthorizationAmount(),
      'authorization_environment' => $transaction->getAuthorizationEnvironment(),
      'authorized_on' => $transaction->getAuthorizedOn(),
      'completed_amount' => $transaction->getCompletedAmount(),
      'completed_on' => $transaction->getCompletedOn(),
      'currency' => $transaction->getCurrency(),
      'state' => $transaction->getState(),
    ];
  }

  /**
   * Create refund for transaction.
   */
  public function createRefund($user_id, $secret, $space_id, $amount, $remote_id) {
    // Create refund object.
    $trans = new \Wallee\Sdk\Model\Transaction();
    $trans->setId($remote_id);

    $refund = new \Wallee\Sdk\Model\Refund();
    $refund->setAmount($amount);
    $refund->setExternalId(time());
    $refund->setMerchantReference('commerce_wallee');
    $refund->setTransaction($trans);
    $refund->setType('CUSTOMER_INITIATED_MANUAL');

    // Setup API client and call refund.
    $client = $this->apiClient($user_id, $secret);
    return $client->getRefundService()->refund($space_id, $refund);
  }

}
